using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;
using GameLibrary;

namespace Entity_Grouping_Demo
{
    public class Button : Sprite 
    {
        public string text;
        private SpriteBatch p_spriteBatch;
        private SpriteFont p_font;
        

        public Button(ContentManager content, SpriteBatch spriteBatch, SpriteFont font) :
            base(content, spriteBatch)
        {
            p_spriteBatch = spriteBatch;
            p_font = font;
            Load("button");
            text = "";
            color = Color.LightGreen;
        }

        public void Draw()
        {
            base.Draw();
            Vector2 size = p_font.MeasureString(text);
            Vector2 pos = position;
            pos.X -= size.X / 2;
            pos.Y -= size.Y / 2;
            p_spriteBatch.DrawString(p_font, text, pos, color);
        }

        public bool Tapped(Vector2 pos)
        {
            Rectangle rect = new Rectangle((int)pos.X, (int)pos.Y, 1, 1);
            return Boundary().Intersects(rect);
        }

    }

    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        TouchLocation oldTouch;
        Random rand;
        SpriteFont font, buttonFont;
        Button plus, minus;
        List<Sprite> cleaners;
        Texture2D circleImage;
        List<Sprite> trash;
        Texture2D trashImage;
        int lastTime = 0;
        int target = -1;
        int respawn = 500;
        int score = 0;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            TargetElapsedTime = TimeSpan.FromTicks(333333);
            oldTouch = new TouchLocation();
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            rand = new Random();
            spriteBatch = new SpriteBatch(GraphicsDevice);
            font = Content.Load<SpriteFont>("WascoSans");
            buttonFont = Content.Load<SpriteFont>("ButtonFont");

            //create minus button
            minus = new Button(Content, spriteBatch, buttonFont);
            minus.position = new Vector2(32, 480-32);
            minus.text = "-";

            //create plus button
            plus = new Button(Content, spriteBatch, buttonFont);
            plus.position = new Vector2(800 - 32, 480 - 32);
            plus.text = "+";

            //create cleaners group
            cleaners = new List<Sprite>();
            circleImage = Content.Load<Texture2D>("circle");
            AddCleaner();

            //create trash group
            trash = new List<Sprite>();
            trashImage = Content.Load<Texture2D>("trash");
            AddTrash();
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            //get state of touch input
            TouchCollection touchInput = TouchPanel.GetState();
            if (touchInput.Count > 0)
            {
                TouchLocation touch = touchInput[0];

                if (touch.State == TouchLocationState.Pressed && 
                    oldTouch.State == TouchLocationState.Released)
                {
                    if (minus.Tapped(touch.Position))
                        RemoveCleaner();
                    if (plus.Tapped(touch.Position))
                        AddCleaner();
                }

                oldTouch = touch;
            }

            //add new trash item periodically
            if ((int)gameTime.TotalGameTime.TotalMilliseconds > lastTime + respawn)
            {
                lastTime = (int)gameTime.TotalGameTime.TotalMilliseconds;
                AddTrash();
            }

            if (trash.Count > cleaners.Count * 2)
            {
                respawn += 1;
                if (respawn > 1000)
                    respawn = 1000;
            }
            else
            {
                respawn -= 1;
                if (respawn < 10)
                    respawn = 10;
            }


            //move the cleaners
            for (int n = 0; n < cleaners.Count; n++)
            {
                //find trash to pick up
                target = FindNearestTrash(cleaners[n].position);
                if (target > -1)
                {
                    float angle = TargetAngle(cleaners[n].position, trash[target].position);
                    cleaners[n].velocityLinear = Velocity(angle, 2.0f);
                    cleaners[n].Move();
                }

                //look for collision with trash
                CollectTrash(cleaners[n].position);

                //look for collision with other cleaners
                for (int c = 0; c < cleaners.Count; c++)
                {
                    if (n != c)
                    {
                        while (cleaners[n].Boundary().Intersects(cleaners[c].Boundary()))
                        {
                            cleaners[c].velocityLinear.X += 0.001f;
                            cleaners[c].velocityLinear.Y += 0.001f;
                            cleaners[c].Move();
                        }
                    }
                }

            }



            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.DarkBlue);
            spriteBatch.Begin();

            minus.Draw();
            plus.Draw();

            foreach (Sprite spr in trash)
                spr.Draw();

            foreach (Sprite spr in cleaners)
                spr.Draw();

            spriteBatch.DrawString(font, "Cleaners:"+cleaners.Count.ToString(), Vector2.Zero, Color.White);
            spriteBatch.DrawString(font, "Trash:"+trash.Count.ToString(), new Vector2(0,25), Color.White);
            spriteBatch.DrawString(font, "Score:" + score.ToString(), new Vector2(0, 50), Color.White);
            spriteBatch.DrawString(font, "Respawn:" + respawn.ToString(), new Vector2(0, 75), Color.White);

            spriteBatch.End();
            base.Draw(gameTime);
        }

        void RemoveCleaner()
        {
            if (cleaners.Count > 0)
                cleaners.RemoveAt(0);
        }

        void AddCleaner()
        {
            Sprite obj = new Sprite(Content, spriteBatch);
            obj.image = circleImage;
            obj.position = new Vector2((float)rand.Next(0, 760), 
                (float)rand.Next(0, 450));
            cleaners.Add(obj);
        }

        void AddTrash()
        {
            Sprite obj = new Sprite(Content, spriteBatch);
            obj.image = trashImage;
            obj.position = new Vector2((float)rand.Next(0, 760),
                (float)rand.Next(0, 450));
            trash.Add(obj);
        }

        int FindNearestTrash(Vector2 pos)
        {
            int target = -1;
            float closest = 9999;

            if (trash.Count == 0) return -1;

            for (int n = 0; n < trash.Count; n++)
            {
                float dist = Distance(pos, trash[n].position);
                if (dist < closest)
                {
                    closest = dist;
                    target = n;
                }
            }
            return target;
        }

        void CollectTrash(Vector2 pos)
        {
            if (trash.Count == 0) return;

            for (int n = 0; n < trash.Count; n++)
            {
                float dist = Distance(pos, trash[n].position);
                if (dist < 8)
                {
                    score++;
                    trash.RemoveAt(n);
                    break;
                }
            }
        }

        float Distance(Vector2 A, Vector2 B)
        {
            double diffX = A.X - B.X;
            double diffY = A.Y - B.Y;
            double dist = Math.Sqrt(Math.Pow(diffX, 2) + Math.Pow(diffY, 2));
            return (float)dist;
        }

        float TargetAngle(Vector2 p1, Vector2 p2)
        {
            return TargetAngle(p1.X, p1.Y, p2.X, p2.Y);
        }

        float TargetAngle(double x1, double y1, double x2, double y2)
        {
            double deltaX = (x2 - x1);
            double deltaY = (y2 - y1);
            return (float)Math.Atan2(deltaY, deltaX);
        }

        Vector2 Velocity(float angle, float acceleration)
        {
            double x = Math.Cos(angle) * acceleration;
            double y = Math.Sin(angle) * acceleration;
            return new Vector2((float)x, (float)y);
        }

    }
}
