using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace GameLibrary
{
    public class Animation
    {
        public bool animating;

        public Animation()
        {
            animating = false;
        }

        public virtual void Update()
        {
        }

        public virtual Color ModifyColor(Color original)
        {
            return original;
        }

        public virtual Vector2 ModifyPosition(Vector2 original)
        {
            return original;
        }

        public virtual float ModifyRotation(float original)
        {
            return original;
        }

        public virtual Vector2 ModifyScale(Vector2 original)
        {
            return original;
        }
    }


    public class SolidColor : Animation
    {
        public Color color;

        public SolidColor(Color color)
            : base()
        {
            this.color = color;
            animating = true;
        }

        public override Color ModifyColor(Color original)
        {
            return color;
        }
    }
 
    public class CycleColor : Animation
    {
        public int red, green, blue, alpha;

        public CycleColor(int red, int green, int blue, int alpha)
            : base()
        {
            this.red = red;
            this.green = green;
            this.blue = blue;
            this.alpha = alpha;
            animating = true;
        }

        public override Color ModifyColor(Color original)
        {
            Color modified = original;

            if (animating)
            {
                int R = original.R + red;
                int G = original.G + green;
                int B = original.B + blue;
                int A = original.A + alpha;

                if (R < 0 || R > 255 || G < 0 || G > 255 || B < 0 || B > 255 || A < 0 || A > 255)
                {
                    animating = false;
                }
                modified = new Color(R, G, B, A);
            }
            return modified;
        }
    }

    public class CycleColorBounce : CycleColor
    {
        public int rmin, rmax, gmin, gmax, bmin, bmax, amin, amax;
        
        public CycleColorBounce(int red, int green, int blue, int alpha)
            : base(red,green,blue,alpha)
        {
            rmin = gmin = bmin = amin = 0;
            rmax = gmax = bmax = amax = 255;
        }

        public override Color ModifyColor(Color original)
        {
            Color modified = original;

            if (animating)
            {
                int R = original.R + red;
                int G = original.G + green;
                int B = original.B + blue;
                int A = original.A + alpha;

                if (R < rmin)
                {
                    R = rmin;
                    red *= -1;
                }
                else if (R > rmax)
                {
                    R = rmax;
                    red *= -1;
                }

                if (G < gmin)
                {
                    G = gmin;
                    green *= -1;
                }
                else if (G > gmax)
                {
                    G = gmax;
                    green *= -1;
                }

                if (B < bmin)
                {
                    B = bmin;
                    blue *= -1;
                }
                else if (B > bmax)
                {
                    B = bmax;
                    blue *= -1;
                }

                if (A < amin)
                {
                    A = amin;
                    alpha *= -1;
                }
                else if (A > amax)
                {
                    A = amax;
                    alpha *= -1;
                }

                modified = new Color(R, G, B, A);
            }
            return modified;
        }
    }


    public class OrbitalMovement : Animation
    {
        public int radius;
        public Vector2 center;
        public double angle;
        public float velocity;

        public OrbitalMovement(Vector2 center, int radius, double angle, float velocity)
            : base()
        {
            animating = true;
            this.center = center;
            this.radius = radius;
            this.angle = angle;
            this.velocity = velocity;
        }

        public override Vector2 ModifyPosition(Vector2 original)
        {
            Vector2 modified = original;

            angle += velocity;

            modified.X = center.X + (float)(Math.Cos(angle) * radius);
            modified.Y = center.Y + (float)(Math.Sin(angle) * radius);

            return modified;
        }

    }

}
