using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace GameLibrary
{
    public class Animation
    {
        public bool animating;

        public Animation()
        {
            animating = false;
        }

        public virtual void Update()
        {
        }

        public virtual Color ModifyColor(Color original)
        {
            return original;
        }

        public virtual Vector2 ModifyPosition(Vector2 original)
        {
            return original;
        }

        public virtual float ModifyRotation(float original)
        {
            return original;
        }

        public virtual Vector2 ModifyScale(Vector2 original)
        {
            return original;
        }

        //
        public virtual int ModifyFrame(int current)
        {
            return current;
        }
    }

    public class FrameLoop : Animation//
    {
        public int start, end, direction;

        public FrameLoop(int start, int end, int direction)
            : base()
        {
            animating = true;
            this.start = start;
            this.end = end;
            this.direction = direction;
        }

        public override int ModifyFrame(int current)
        {
            current += direction;
            if (current > end)
                current = start;
            else if (current < start)
                current = end;

            return current;
        }
    }

    public class FrameBounce : Animation//
    {
        public int start, end, direction;

        public FrameBounce(int start, int end, int direction)
            : base()
        {
            animating = true;
            this.start = start;
            this.end = end;
            this.direction = direction;
        }

        public override int ModifyFrame(int current)
        {
            current += direction;
            if (current > end)
            {
                direction *= -1;
                current = end;
            }
            else if (current < start)
            {
                direction *= -1;
                current = start;
            }

            return current;
        }
    }


    public class SolidColor : Animation
    {
        public Color color;

        public SolidColor(Color color)
            : base()
        {
            this.color = color;
            animating = true;
        }

        public override Color ModifyColor(Color original)
        {
            return color;
        }
    }
 
    public class CycleColor : Animation
    {
        public int red, green, blue, alpha;

        public CycleColor(int red, int green, int blue, int alpha)
            : base()
        {
            this.red = red;
            this.green = green;
            this.blue = blue;
            this.alpha = alpha;
            animating = true;
        }

        public override Color ModifyColor(Color original)
        {
            Color modified = original;

            if (animating)
            {
                int R = original.R + red;
                int G = original.G + green;
                int B = original.B + blue;
                int A = original.A + alpha;

                if (R < 0 || R > 255 || G < 0 || G > 255 || B < 0 || B > 255 || A < 0 || A > 255)
                {
                    animating = false;
                }
                modified = new Color(R, G, B, A);
            }
            return modified;
        }
    }

    public class CycleColorBounce : CycleColor
    {
        public int rmin, rmax, gmin, gmax, bmin, bmax, amin, amax;
        
        public CycleColorBounce(int red, int green, int blue, int alpha)
            : base(red,green,blue,alpha)
        {
            rmin = gmin = bmin = amin = 0;
            rmax = gmax = bmax = amax = 255;
        }

        public override Color ModifyColor(Color original)
        {
            Color modified = original;

            if (animating)
            {
                int R = original.R + red;
                int G = original.G + green;
                int B = original.B + blue;
                int A = original.A + alpha;

                if (R < rmin)
                {
                    R = rmin;
                    red *= -1;
                }
                else if (R > rmax)
                {
                    R = rmax;
                    red *= -1;
                }

                if (G < gmin)
                {
                    G = gmin;
                    green *= -1;
                }
                else if (G > gmax)
                {
                    G = gmax;
                    green *= -1;
                }

                if (B < bmin)
                {
                    B = bmin;
                    blue *= -1;
                }
                else if (B > bmax)
                {
                    B = bmax;
                    blue *= -1;
                }

                if (A < amin)
                {
                    A = amin;
                    alpha *= -1;
                }
                else if (A > amax)
                {
                    A = amax;
                    alpha *= -1;
                }

                modified = new Color(R, G, B, A);
            }
            return modified;
        }
    }


    public class OrbitalMovement : Animation
    {
        public int radius;
        public Vector2 center;
        public double angle;
        public float velocity;

        public OrbitalMovement(Vector2 center, int radius, double angle, float velocity)
            : base()
        {
            animating = true;
            this.center = center;
            this.radius = radius;
            this.angle = angle;
            this.velocity = velocity;
        }

        public override Vector2 ModifyPosition(Vector2 original)
        {
            Vector2 modified = original;

            angle += velocity;

            modified.X = center.X + (float)(Math.Cos(angle) * radius);
            modified.Y = center.Y + (float)(Math.Sin(angle) * radius);

            return modified;
        }

    }

    public class ClockHand : Animation
    {
        public int direction;

        public ClockHand(int direction) //0 to 59
            : base()
        {
            animating = true;
            this.direction = direction;
        }

        public override float ModifyRotation(float original)
        {
            float angle = (float)(direction / 60.0f) * (float)(2.0f * Math.PI);
            return angle;
        }
    }

    public class Spin : Animation
    {
        private float angleDist, velocity;

        public Spin(float velocity)
            : base()
        {
            animating = true;
            this.velocity = velocity;
            angleDist = 0.0f;
        }

        public override float ModifyRotation(float original)
        {
            if (animating)
            {
                float fullCircle = (float)(2.0 * Math.PI);
                angleDist += velocity;
                if (angleDist > fullCircle)
                    animating = false;

                original += velocity;
            }
            return original;
        }
    }

    public class Throb : Animation
    {
        public float startScale, endScale, speed;
        private bool p_started;

        public Throb(float startScale, float endScale, float speed)
            : base()
        {
            p_started = false;
            animating = true;
            this.startScale = startScale;
            this.endScale = endScale;
            this.speed = speed;
        }

        public override Vector2 ModifyScale(Vector2 original)
        {
            if (!animating) return original;

            Vector2 modified = original;
            
            if (!p_started)
            {
                modified.X = startScale;
                modified.Y = startScale;
                p_started = true;
            }

            modified.X += speed;
            modified.Y += speed;

            if (modified.X >= endScale)
                speed *= -1;
            else if (modified.X <= startScale)
                animating = false;

            return modified;
        }
    }

    public class ThrobBounce : Throb
    {
        public ThrobBounce(float startScale, float endScale, float speed)
            : base(startScale, endScale, speed)
        {
        }

        public override Vector2 ModifyScale(Vector2 original)
        {
            //keep it going
            Vector2 scale = base.ModifyScale(original);
            if (!animating)
            {
                animating = true;
                speed *= -1;
            }
            return scale;
        }

    }

}
