using System;
using System.Collections.Generic;
using System.Device.Location;
using System.Threading;

namespace Geo_Position_Demo
{

    abstract public class GeoLocationSim : GeoCoordinateWatcher
    {
        private GeoPosition<GeoCoordinate> current;
        private Timer timer;

        public GeoLocationSim()
        {
            current = new GeoPosition<GeoCoordinate>();
            Status = GeoPositionStatus.Initializing;
            RaiseStatusChanged();
        }

        private void RaiseStatusChanged()
        {
            GeoPositionStatusChangedEventArgs args = 
                new GeoPositionStatusChangedEventArgs(Status);
            if (StatusChanged != null)
            {
                StatusChanged(this, args);
            }
        }

        private void RaisePositionChanged()
        {
            GeoPositionChangedEventArgs<GeoCoordinate> args = 
                new GeoPositionChangedEventArgs<GeoCoordinate>(current);
            if (PositionChanged != null)
                PositionChanged(this, args);
        }

        public void OnTimerCallback(object state)
        {
            try
            {
                if (Status == GeoPositionStatus.Initializing)
                {
                    Status = GeoPositionStatus.NoData;
                    RaiseStatusChanged();
                }
                StartGetCurrentPosition();
                TimeSpan next = GetNextInterval();
                timer.Change(next, next);
            }
            catch (Exception)
            {
                throw;
            }
        }

        protected void UpdateLocation(double longitude, double latitude)
        {
            GeoCoordinate location = new GeoCoordinate(latitude, longitude);
            if (!location.Equals(current.Location))
            {
                current = new GeoPosition<GeoCoordinate>(
                    DateTimeOffset.Now, location);
                if (Status != GeoPositionStatus.Ready)
                {
                    Status = GeoPositionStatus.Ready;
                    RaiseStatusChanged();
                }
                RaisePositionChanged();
            }
        }

        abstract protected TimeSpan GetNextInterval();
        abstract protected void StartGetCurrentPosition();

        //override base property
        public GeoPositionPermission Permission
        {
            get { return GeoPositionPermission.Granted; }
        }

        //override base property
        public GeoPosition<GeoCoordinate> Position
        {
            get { return current; }
        }

        //override base event
        public event EventHandler<GeoPositionChangedEventArgs
            <GeoCoordinate>> PositionChanged;

        //override base method
        public void Start(bool suppressPermissionPrompt)
        {
            Start();
        }

        //override base method
        public void Start()
        {
            TimeSpan span = GetNextInterval();
            timer = new Timer(OnTimerCallback, null, span, span);
        }

        //override base property
        public GeoPositionStatus Status
        {
            get;
            protected set;
        }

        //override base event
        public event EventHandler
            <GeoPositionStatusChangedEventArgs> StatusChanged;

        //override base method
        public void Stop()
        {
            timer.Change(Timeout.Infinite, Timeout.Infinite);
            Status = GeoPositionStatus.Disabled;
            RaiseStatusChanged();
        }

        //override base method
        public bool TryStart(bool suppressPermissionPrompt, TimeSpan timeout)
        {
            Start();
            return true;
        }
    }

    public class SampleGeoCoord
    {
        public double Longitude { get; set; }
        public double Latitude { get; set; }
        public TimeSpan Time { get; set; }

        public SampleGeoCoord(double Longitude, double Latitude, int seconds)
        {
            this.Longitude = Longitude;
            this.Latitude = Latitude;
            this.Time = new TimeSpan(0, 0, seconds);
        }
    }

    public class GeoCoordinateWatcherSim : GeoLocationSim
    {
        List<SampleGeoCoord> events;
        int currentEventId;
        Random rand = new Random();

        public GeoCoordinateWatcherSim(GeoPositionAccuracy accuracy)
        {
            currentEventId = 0;
            events = new List<SampleGeoCoord>();

            //create random coordinates in Los Angeles
            for (int n = 1; n < 60; n++)
            {
                double Long = -118 - rand.Next(2) - rand.NextDouble();
                double Lat = 33 + rand.Next(2) + rand.NextDouble();
                events.Add(new SampleGeoCoord(Long, Lat, n));
            }
        }

        private SampleGeoCoord Current
        {
            get
            {
                return events[currentEventId % events.Count];
            }
        }

        protected override void StartGetCurrentPosition()
        {
            this.UpdateLocation(Current.Longitude, Current.Latitude);
            currentEventId++;
        }

        protected override TimeSpan GetNextInterval()
        {
            return Current.Time;
        }
    }
}

