using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace GameLibrary
{
    public class Sprite
    {
        protected ContentManager p_content;
        protected SpriteBatch p_spriteBatch;
        public Texture2D image;
        public Vector2 position;
        public Vector2 velocityLinear;
        public Color color;
        public float rotation;
        public float velocityAngular;
        public Vector2 scaleV;
        public Vector2 origin;
        public bool alive;
        public bool visible;
        public List<Animation> animations;
        private double startTime;
        private Vector2 p_size; 
        public int columns, frame, totalFrames, frameDir;
        public float zindex;


        public Sprite(ContentManager content, SpriteBatch spriteBatch)
        {
            p_content = content;
            p_spriteBatch = spriteBatch;
            image = null;
            position = Vector2.Zero;
            velocityLinear = Vector2.Zero;
            color = Color.White;
            rotation = 0.0f;
            velocityAngular = 0.0f;
            scaleV = new Vector2(1.0f);
            origin = Vector2.Zero;
            alive = true;
            visible = true;
            animations = new List<Animation>();
            size = Vector2.Zero;
            columns = 1;
            frame = 0;
            totalFrames = 1;
            frameDir = 1;
            startTime = 0;
            zindex = 0.0f;
        }

        public float X
        {
            get
            {
                return position.X;
            }
            set
            {
                position.X = value;
            }
        }

        public float Y
        {
            get
            {
                return position.Y;
            }
            set
            {
                position.Y = value;
            }
        }

        public Vector2 size
        {
            get { return p_size; }
            set 
            {
                p_size = value;
                origin = new Vector2(p_size.X / 2, p_size.Y / 2);
            }
        }

        public float scale
        {
            get { return scaleV.X; }
            set
            {
                scaleV.X = value;
                scaleV.Y = value;
            }
        }

        public virtual bool Load(string assetName)
        {
            try
            {
                image = p_content.Load<Texture2D>(assetName);
            }
            catch (Exception) { return false; }

            size = new Vector2(image.Width, image.Height);
            origin = new Vector2(image.Width / 2, image.Height / 2);
            
            return true;
        }

        public virtual void Draw()
        {
            if (!visible) return;

            if (image == null) return;
            
            if (totalFrames > 1)
            {
                Rectangle source = new Rectangle();
                source.X = (frame % columns) * (int)size.X;
                source.Y = (frame / columns) * (int)size.Y;
                source.Width = (int)size.X;
                source.Height = (int)size.Y;
                p_spriteBatch.Draw(image, position, source, color,
                    rotation, origin, scale, SpriteEffects.None, zindex);
            }
            else
            {
                p_spriteBatch.Draw(image, position, null, color, rotation,
                    origin, scaleV, SpriteEffects.None, zindex);
            }
        }

        public void Move()
        {
            position += velocityLinear;
        }

        public void Rotate()
        {
            if (velocityAngular != 0.0f)
            {
                rotation += velocityAngular;
                if (rotation > Math.PI * 2)
                    rotation -= (float)Math.PI * 2;
                else if (rotation < 0.0f)
                    rotation = (float)Math.PI * 2 - rotation;
            }
        }

        public Rectangle Boundary()
        {
            int halfw = (int)((float)(size.X / 2) * scaleV.X);
            int halfh = (int)((float)(size.Y / 2) * scaleV.Y);

            return new Rectangle(
                (int)position.X - halfw,
                (int)position.Y - halfh,
                halfw * 2,
                halfh * 2);
        }

        public override string ToString()
        {
            string s = "Texture {W:" + image.Width.ToString() +
                " H:" + image.Height.ToString() + "}\n" +
                "Position {X:" + position.X.ToString("N2") + " Y:" +
                    position.Y.ToString("N2") + "}\n" +
                "Lin Vel " + velocityLinear.ToString() + "\n" +
                "Ang Vel {" + velocityAngular.ToString("N2") + "}\n" +
                "Scaling " + scaleV.ToString() + "\n" +
                "Rotation " + rotation.ToString() + "\n" +
                "Pivot " + origin.ToString() + "\n";
            Rectangle B = Boundary();
            s += "Boundary {X:" + B.X.ToString() + " Y:" +
                B.Y.ToString() + " W:" + B.Width.ToString() +
                " H:" + B.Height.ToString() + "}\n";

            return s;
        }

        public void Animate()
        {
            if (animations.Count == 0) return;

            foreach (Animation anim in animations)
            {
                if (anim.animating)
                {
                    color = anim.ModifyColor(color);
                    position = anim.ModifyPosition(position);
                    rotation = anim.ModifyRotation(rotation);
                    scaleV = anim.ModifyScale(scaleV);
                    frame = anim.ModifyFrame(frame);
                }
                else
                {
                    animations.Remove(anim);
                    return;
                }
            }
        }

        public void Animate(double elapsedTime)
        {
            Animate(0, totalFrames-1, elapsedTime, 30);
        }

        public void Animate(int startFrame, int endFrame, double elapsedTime, double speed)
        {
            if (totalFrames <= 1) return;

            startTime += elapsedTime;
            if (startTime > speed)
            {
                startTime = 0;
                if (++frame > endFrame) frame = startFrame;
            }
        }


    }
}
