using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;
using GameLibrary;

namespace Black_Hole_Game
{
    public class PlayingModule : IGameModule
    {
        Game1 game;
        Button btnFire, btnQuit;
        Label lblAngle, lblPower;
        HSlider hsAngle, hsPower;
        Random rand;
        Sprite background;
        MassiveObject blackHole;
        MassiveObject superCore;
        MassiveObject ship;
        float energy;
        int startTime, lifetime;
        MySoundEffect launchSound;
        int lastLaunch, launchTime;

        List<MassiveObject> objects;

        public PlayingModule(Game1 game)
        {
            this.game = game;
            rand = new Random();

            objects = new List<MassiveObject>();

            ship = new MassiveObject(game.Content, game.spriteBatch);
            blackHole = new MassiveObject(game.Content, game.spriteBatch);
            superCore = new MassiveObject(game.Content, game.spriteBatch);
            hsAngle = new HSlider(game.Content, game.spriteBatch, game.guifont);
            lblAngle = new Label(game.Content, game.spriteBatch, game.guifont);
            hsPower = new HSlider(game.Content, game.spriteBatch, game.guifont);
            lblPower = new Label(game.Content, game.spriteBatch, game.guifont);
            btnFire = new Button(game.Content, game.spriteBatch, game.guifont);
            btnQuit = new Button(game.Content, game.spriteBatch, game.guifont);

            Reset();
        }

        public void Reset()
        {
            startTime = 0;
            lifetime = 4000;
            lastLaunch = 0;
            launchTime = 2000;
            energy = 100.0f;
            ship.position = new Vector2(200, 240);
            hsAngle.Value = 30;
            hsPower.Value = 50;
            ship.lifetime = 0;
            ship.radius = 250;
            ship.angle = MathHelper.ToRadians(180);
            ship.rotation = MathHelper.ToRadians(90);
            objects.Clear();
        }

        public void LoadContent(ContentManager content)
        {
            launchSound = new MySoundEffect(game);
            launchSound.Load("launch");

            background = new Sprite(game.Content, game.spriteBatch);
            background.Load("space");
            background.origin = Vector2.Zero;

            blackHole.Load("blackhole");
            blackHole.position = new Vector2(400, 240);
            blackHole.scale = 2.0f;
            blackHole.mass = 40;
            blackHole.color = new Color(255, 100, 100, 210);
            blackHole.velocityAngular = 0.1f;

            superCore.image = blackHole.image;
            superCore.position = new Vector2(blackHole.position.X, 
                blackHole.position.Y);
            superCore.scale = blackHole.scale * 0.4f;
            superCore.mass = 60;
            superCore.color = new Color(200, 100, 100, 190);
            superCore.velocityAngular = 4.0f;
            superCore.origin = new Vector2(64, 64);

            //player ship
            ship.Load("ship");
            ship.mass = 20f;
            ship.scale = 0.2f;

            //angle slider
            hsAngle.SetStartPosition(new Vector2(170, 445));
            hsAngle.color = Color.Orange;
            hsAngle.Limit = 108;

            //angle label
            lblAngle.position = new Vector2(hsAngle.X, hsAngle.Y-40);
            lblAngle.text = "ANGLE";

            //power slider
            hsPower.SetStartPosition(new Vector2(530, 445));
            hsPower.color = Color.Orange;
            hsPower.Limit = 100;

            //power label
            lblPower.position = new Vector2(hsPower.X, hsPower.Y-40);
            lblPower.text = "POWER";

            //fire button
            btnFire.position = new Vector2(400, 440);
            btnFire.color = Color.Orange;
            btnFire.UseShadow = false;
            btnFire.text = "LAUNCH";
            btnFire.scaleV = new Vector2(1.5f, 0.7f);

            //quit button
            btnQuit.text = "X";
            btnQuit.position = new Vector2(800 - 20, 480 - 20);
            btnQuit.scaleV = new Vector2(0.3f, 0.5f);
        }

        public void Update(TouchLocation touch, GameTime gameTime)
        {
            //update user controls
            btnFire.Update(touch);
            hsAngle.Update(touch);
            hsPower.Update(touch);
            lblAngle.Update(touch);
            lblPower.Update(touch);
            btnQuit.Update(touch);

            //update gameplay objects
            blackHole.Update(gameTime);
            blackHole.Rotate();
            superCore.Update(gameTime);
            superCore.Rotate();
            UpdateObjects(gameTime);
            UpdateShip(gameTime);

            //check user input
            if (btnFire.Tapped)
            {
                if (lastLaunch + launchTime < gameTime.TotalGameTime.
                    TotalMilliseconds)
                {
                    lastLaunch = (int)gameTime.TotalGameTime.
                        TotalMilliseconds;
                    launchSound.Play(); 
                    CreateSatellite();
                }
            }

            //rotate ship with slider
            float angle = hsAngle.Value * 3.3f;
            ship.rotation = MathHelper.ToRadians(angle);
            lblAngle.text = "ANGLE:" + angle.ToString("N0");

            //set power label
            lblPower.text = "POWER:" + hsPower.Value.ToString();

            //time to add another random asteroid?
            if (startTime + lifetime < gameTime.TotalGameTime.
                TotalMilliseconds)
            {
                startTime = (int)gameTime.TotalGameTime.TotalMilliseconds;
                CreateAsteroid();
            }

            //user quit?
            if (btnQuit.Tapped)
            {
                game.gameState = Game1.GameState.TITLE;
            }
   
        }

        public void UpdateObjects(GameTime gameTime)
        {
            int time = gameTime.ElapsedGameTime.Milliseconds;

            foreach (MassiveObject obj in objects)
            {
                if (!obj.alive) continue;

                obj.Update(gameTime);
                obj.Rotate();
                obj.Animate(time);
                obj.Animate();

                //allow ship to collect energy satellite
                if (obj.scale >= 1.0f && obj.name == "satellite")
                {
                    //when large, cause satellites to move toward the ship
                    obj.Attract(ship);

                    //reset satellite to white when seeking ship
                    obj.color = Color.White;

                    //look for collision with ship
                    if (game.RadialCollision(obj.position, ship.position, obj.size.X, 40))
                    {
                        obj.alive = false;
                        energy += obj.scale * 4;
                        if (energy > 200) energy = 200;
                    }
                }

                if (!obj.captured)
                {
                    //only attract when object is near the black hole
                    if (game.RadialCollision(obj.position, blackHole.position, obj.size.X, 500))
                    {
                        obj.Attract(blackHole);
                        //obj.Attract(superCore);

                        //is object touching the outer edges of the black hole?
                        if (game.RadialCollision(obj.position, blackHole.position, obj.size.X, 120))
                        {
                            //turn red when going through inner gravity well
                            obj.color = Color.Red;

                            if (obj.name == "satellite")
                            {
                                obj.scale += 0.1f;
                                if (obj.scale > 5.0f) obj.scale = 5.0f;
                                energy += 0.1f;
                                if (energy > 200) energy = 200;
                            }

                            //obj.Attract(blackHole);
                            obj.Attract(superCore);

                            //uh oh, object is caught by the black hole
                            if (game.RadialCollision(obj.position, superCore.position, 16, 60))
                            {
                                obj.captured = true;
                                //set a lifetime delay once captured
                                obj.lifetime = 3000;
                                obj.startTime = (int)gameTime.TotalGameTime.TotalMilliseconds;
                                //cause object to spin around the black hole when captured
                                OrbitalMovement anim1 = new OrbitalMovement(blackHole.position,
                                    10 + rand.Next(40), obj.rotation, -0.8f);
                                obj.animations.Add(anim1);
                            }
                        }
                        else
                        {
                            obj.color = Color.White;
                        }
                    }
                }

                //when captured, time runs out
                if (obj.lifetime > 0)
                {
                    if (obj.startTime + obj.lifetime < gameTime.TotalGameTime.TotalMilliseconds)
                        obj.alive = false;
                }

                //see if object has gone too far out of bounds
                if (obj.position.X < -200 || obj.position.X > 1000 ||
                    obj.position.Y < -200 || obj.position.Y > 700)
                    obj.alive = false;

            }
        }

        public void UpdateShip(GameTime gameTime)
        {
            int time = gameTime.ElapsedGameTime.Milliseconds;

            ship.Update(gameTime);
            ship.Rotate();
            ship.Animate();

            //cause ship to fall into black hole
            if (!ship.captured)
            {
                //update ship position
                ship.X = 400 + (float)(Math.Cos(ship.angle) * ship.radius);
                ship.Y = 240 + (float)(Math.Sin(ship.angle) * ship.radius);

                //consume energy
                energy -= 0.05f;

                if (energy > 0)
                {
                    //while we have energy, try to get away
                    ship.radius += 0.2f;
                    if (ship.radius > 250)
                        ship.radius = 250;
                }

                if (energy < 50)
                {
                    //rotate ship around black hole (custom)
                    ship.angle += 0.01f;

                    if (energy <= 0)
                    {
                        energy = 0;
                        ship.radius -= 0.1f;

                        //uh oh, ship is caught by the black hole
                        if (game.RadialCollision(ship.position, 
                            superCore.position, 64, 40))
                        {
                            ship.captured = true;
                            ship.velocityAngular = 1.0f;
                            ship.lifetime = 5000;
                            ship.startTime = (int)gameTime.TotalGameTime.
                                TotalMilliseconds;
                            OrbitalMovement anim1 = new OrbitalMovement(
                                blackHole.position, 10 + rand.Next(40), 
                                0, 0.4f);
                            ship.animations.Add(anim1);
                        }
                    }
                }

            }


            //ship fell into black hole?
            if (ship.lifetime > 0)
            {
                if (ship.startTime + ship.lifetime < gameTime.TotalGameTime.
                    TotalMilliseconds)
                {
                    game.gameState = Game1.GameState.GAMEOVER;
                    return;
                }
            }

        }


        public void Draw(GameTime gameTime)
        {
            background.Draw();

            superCore.Draw();
            blackHole.Draw();
            ship.Draw();

            foreach (MassiveObject obj in objects)
            {
                if (obj.alive)
                {
                    obj.Draw();
                }
            }

            btnFire.Draw();
            hsAngle.Draw();
            hsPower.Draw();
            lblAngle.Draw();
            lblPower.Draw();
            btnQuit.Draw();

            string text;
            //text = "ship.rotation " + MathHelper.ToDegrees(ship.rotation).ToString("N0");
            //game.spriteBatch.DrawString(game.font, text, new Vector2(0, 0), Color.White);

            //text = "ship.angle " + MathHelper.ToDegrees(ship.angle).ToString("N0");
            //game.spriteBatch.DrawString(game.font, text, new Vector2(0, 20), Color.White);

            //text = "ship.radius " + ship.radius.ToString("N0");
            //game.spriteBatch.DrawString(game.font, text, new Vector2(0, 40), Color.White);

            text = "Energy " + energy.ToString("N0");
            game.spriteBatch.DrawString(game.font, text, new Vector2(650, 0), Color.White);
        }


        public void CreateAsteroid()
        {
            MassiveObject obj = new MassiveObject(game.Content, 
                game.spriteBatch);
            obj.Load("asteroid");
            obj.columns = 8;
            obj.totalFrames = 64;
            obj.scale = 0.1f + (float)rand.NextDouble();
            obj.size = new Vector2(60, 60);
            obj.radius = 80;

            //randomly place at top or bottom of screen
            obj.position = new Vector2(rand.Next(100, 800), -100);
            obj.velocityLinear = new Vector2(4.0f, (float)
                (rand.NextDouble() * 6.0));
            if (rand.Next(2) == 1)
            {
                obj.position.Y = -obj.position.Y;
                obj.velocityLinear.Y = -obj.velocityLinear.Y;
            }

            obj.scale = (0.5f + (float)rand.NextDouble()) * 0.5f;
            obj.mass = 1;
            obj.velocityAngular = 0.001f;
            obj.lifetime = 0;
            obj.name = "asteroid";
            objects.Add(obj);
        }

        public void CreateSatellite() 
        {
            MassiveObject obj;
            obj = new MassiveObject(game.Content, game.spriteBatch);
            obj.Load("plasma32");
            obj.position = ship.position;
            obj.mass = 1;
            obj.scale = 0.5f;
            obj.lifetime = 0;
            obj.name = "satellite";

            //calculate velocity based on ship's angle
            float accel = 1 + (float)(hsPower.Value / 10);
            float angle = ship.rotation - MathHelper.ToRadians(90);
            float x = (float)Math.Cos(angle) * accel;
            float y = (float)Math.Sin(angle) * accel;
            obj.velocityLinear = new Vector2(x,y);

            //use energy to launch satellite
            energy -= 1;

            objects.Add(obj);
        }

    }
}
